@@ -1,7 +1,6 @@
 /*
- * @(#)EdgeRenderer.java	1.0 03-JUL-04
- * 
- * Copyright (c) 2001-2005 Gaudenz Alder
+ * $Id: EdgeRenderer.java,v 1.18 2008/05/08 12:57:35 david Exp $ * 
+ * Copyright (c) 2001-2007 Gaudenz Alder
  *  
  * See LICENSE file in distribution for licensing details of this source file
  */
@@ -60,6 +59,12 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 		}
 	}
 
+	/** When zooming a graph the font size jumps at certain zoom levels rather than
+	 *  scaling smoothly. Sometimes the zoom on the font is more than the component zoom
+	 *  and cropping occurs. This buffer allows for the maximum occurance of this
+	 */
+	public static double LABELWIDTHBUFFER = 1.1;
+	
 	/** A switch for painting the extra labels */
 	public boolean simpleExtraLabels = true;
 
@@ -117,13 +122,11 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 	/** The color of highlighted cells */
 	protected transient Color highlightColor = null;
 
-	/* THIS CODE WAS ADDED BY MARTIN KRUEGER 10/20/2003 */
+	/** Cached bezier curve */
+	protected transient Bezier bezier;
 
-	protected Bezier bezier;
-
-	protected Spline2D spline;
-
-	/* END */
+	/** Cached spline curve */
+	protected transient Spline2D spline;
 
 	/**
 	 * Constructs a renderer that may be used to render edges.
@@ -438,7 +441,7 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 		Point2D p0 = view.getPoint(0);
 		if (pos != null && tmp != null) {
 			if (!isLabelTransformEnabled()) {
-				return getRelativeLabelPosition(view, pos);
+				return view.getAbsoluteLabelPositionFromRelative(pos);
 			} else {
 				Point2D vector = view.getLabelVector();
 				double dx = vector.getX();
@@ -458,81 +461,6 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 		}
 		return null;
 	}
-	
-	protected Point2D getRelativeLabelPosition(EdgeView edge, Point2D geometry) {
-		int pointCount = edge.getPointCount();
-		
-		double length = 0;
-		double[] segments = new double[pointCount];
-		Point2D pt = edge.getPoint(0);
-		
-		if (pt != null)
-		{
-			for (int i = 1; i < pointCount; i++)
-			{
-				Point2D tmp = edge.getPoint(i);
-				
-				if (tmp != null)
-				{
-					double dx = pt.getX() - tmp.getX();
-					double dy = pt.getY() - tmp.getY();
-
-					double segment = Math.sqrt(dx * dx + dy * dy);
-					
-					segments[i - 1] = segment;
-					length += segment;
-					pt = tmp;
-				}
-			}
-
-			double x = geometry.getX()/GraphConstants.PERMILLE;
-			double y = geometry.getY();
-			
-			double dist = x * length;
-			length = 0;
-			
-			int index = 1;
-			double segment = segments[0];
-
-			while (dist > length + segment && index < pointCount - 1)
-			{
-				length += segment;
-				segment = segments[index++];
-			}
-
-			double factor = (dist - length) / segment;
-			
-			Point2D p0 = edge.getPoint(index - 1);
-			Point2D pe = edge.getPoint(index);
-
-			if (p0 != null && pe != null)
-			{
-				double dx = pe.getX() - p0.getX();
-				double dy = pe.getY() - p0.getY();
-				
-				double nx = dy / segment;
-				double ny = dx / segment;
-				
-				double offsetX = 0;
-				double offsetY = 0;
-				
-				Point2D offset = GraphConstants.getOffset(edge.getAllAttributes());
-				
-				if (offset != null)
-				{
-					offsetX = offset.getX();
-					offsetY = offset.getY();
-				}
-				
-				x = p0.getX() + dx * factor + nx * y + offsetX;
-				y = p0.getY() + dy * factor + ny * y + offsetY;
-				
-				return new Point2D.Double(x, y);
-			}
-		}
-		
-		return null;
-	}
 
 	/**
 	 * Returns the label size of the specified view in the given graph.
@@ -558,7 +486,7 @@ public class EdgeRenderer extends JComponent implements CellViewRenderer,
 			fontGraphics.setFont(GraphConstants
 					.getFont(view.getAllAttributes()));
 			metrics = fontGraphics.getFontMetrics();
-			int sw = metrics.stringWidth(label);
+			int sw = (int)(metrics.stringWidth(label) * LABELWIDTHBUFFER);
 			int sh = metrics.getHeight();
 			return new Dimension(sw, sh);
 		}
